%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Purpose
%
%       Driver for Example 17.1. 
%
%       See Example 17.1 in J. Nocedal and S. J. Wright,
%       "Numerical Optimization", Springer-Verlag, 1999
%
%  Functions called: func_c.m, func_f.m, gradient_f.m, hessian_c.m,   
%                    hessian_f.m, jacobian_c.m
%
%
%  Written by L. N. Vicente.
%  May 25, 2000.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Begin.
%
% Clear work space and screen.
  clf;
  clear;
%
% Set formats.
  format compact;
  format long;
%
% Initializations.
%
% fid   file identifier for output file. No output if fid < 0.
%       Use fid = 1 for standard output (usually screen).
  fid = 1; 
%
% Initial points.
%
  x0     = [-1.2 ; -1.2];
  sol    = [-1.0 ; -1.0];
%
%
%  
% Beginning of quadratic penalty method.
  fprintf( fid,'\n Quadratic penalty method with Newton''s method for inner iterations: \n');
  mu_seq = [10^(-1) ; 10^(-2) ; 10^(-3) ; 10^(-5) ; 10^(-7) ; 10^(-9) ; 10^(-11)];
  ni     = size(mu_seq,1);
  x      = x0;    
%
% Prepare gradient of quadratic penalty function at x=x0.
  [gf] = gradient_f(x);
  [c]  = func_c(x);
  [A]  = jacobian_c(x);
%
  for i = 1:ni 
%
% Update penalty parameter.
  mu = mu_seq(i);
%
% Compute gradient of quadratic penalty function at x.
  g  = gf + 1/mu*A'*c;
%
% Newton's method for inner iteration.
  iter = 0; ifunc=0; ig=1; iH=0;
% Initial point is equal to last inner iteration point.
  while ( norm(g) > sqrt(eps) & iter <= 99 )
          [Hf] = hessian_f(x);
          [Hc] = hessian_c(x);
        % Compute Hessian of quadratic penalty function at x.
          H    = Hf + c/mu*Hc + 1/mu*A'*A;
          iH   = iH+1;
%
   x = x - H\g;
%
%  Modification 17.18. 
%  C = [ Hf+c/mu*Hc  A'
%        A          -mu ];
%  b = [ g ; 0 ];
%  y = - C\b;
%  x = x + y(1:2);
%  End of modification 17.18.
%        
          [gf] = gradient_f(x);
          [c]  = func_c(x);
          [A]  = jacobian_c(x);
        % Compute gradient of quadratic penalty function at x.
          g    = gf + 1/mu*A'*c;
          ig   = ig+1;
          iter = iter + 1;
  end
%
  normg = norm(g);
% 
   condH = cond(H);
%  Modification 17.18.
%   condH = cond(C);
%  End of modification 17.18.
%
  dist  = norm(x-sol);
%
% Printing.
  fprintf( fid,'\n|    mu    |  iter  | fQeval | gQeval | HQeval |  norm(gQ)  |  cond(HQ)  | norm(x-s)  | \n');
  format_str = '| %7.2e | %6d | %6d | %6d | %6d | %9.4e | %9.4e | %9.4e | \n';
  fprintf( fid, format_str, mu, iter, ifunc, ig, iH, normg, condH, dist);
%
  end   
% End of quadratic penalty method.
  fprintf( fid,'\n');
%
%
%
% Beginning of augmented Lagrangian method.
  fprintf( fid,'\n Augmented Lagrangian method with Newton''s method for inner iterations: \n');
  mu_seq = [10^(-1) ; 10^(-2) ; 10^(-3) ; 10^(-3) ; 10^(-3) ; 10^(-3) ; 10^(-3)];
  ni     = size(mu_seq,1);
  x      = x0;
  lambda = 0;
%
% Prepare gradient of augmented Lagrangian at x=x0.
  [gf] = gradient_f(x);
  [c]  = func_c(x);
  [A]  = jacobian_c(x);
%
  for i = 1:ni
%
% Update penalty parameter and Lagrange multipliers.
  mu     = mu_seq(i);
  lambda = lambda - c/mu;
%
% Compute gradient of augmented Lagrangian at x.
  g = gf - A'*lambda + 1/mu*A'*c;
%
% Newton's method for inner iteration.
  iter = 0; ifunc=0; ig=1; iH=0;
% Initial point is equal to last inner iteration point.
  while ( norm(g) > sqrt(eps) & iter <= 99 )
          [Hf] = hessian_f(x);
          [Hc] = hessian_c(x);
        % Compute Hessian of augmented Lagrangian at x.
          H    = Hf - lambda*Hc + 1/mu*(c*Hc+A'*A);
          iH   = iH+1;
          x    = x - H\g;
          [gf] = gradient_f(x);
          [c]  = func_c(x);
          [A]  = jacobian_c(x);
        % Compute gradient of augmented Lagrangian at x.
          g    = gf - A'*lambda + 1/mu*A'*c;
          ig   = ig+1;
          iter = iter + 1;
  end
%
  normg = norm(g);
  condH = cond(H);
  dist  = norm(x-sol);
%
% Printing.
  fprintf( fid,'\n|    mu    |  iter  | fLeval | gLeval | HLeval |  norm(gL)  |  cond(HL)  | norm(x-s)  |  \n');
  format_str = '| %7.2e | %6d | %6d | %6d | %6d | %9.4e | %9.4e | %9.4e | \n';
  fprintf( fid, format_str, mu, iter, ifunc, ig, iH, normg, condH, dist);
%
  end   
% End of augmented Lagrangian method.
  fprintf( fid,'\n');
%
% End of driver for Example 17.1 of Nocedal and Wright.
